#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Base proxy model for OCD campaign_finance related managers.
"""
from __future__ import unicode_literals

# Database
from psycopg2 import sql
from django.db import connection

# Managers
from calaccess_processed.managers import BulkLoadSQLManager

# Text
import re
from django.template.defaultfilters import pluralize

# Logging
import logging
logger = logging.getLogger(__name__)


class CampaignFinanceManager(BulkLoadSQLManager):
    """
    Base proxy model for OCD campaign_finance related managers.
    """
    app_name = "calaccess_processed_campaignfinance"

    def get_sql(self, file_name):
        """
        Return string of raw sql for loading the model.
        """
        sql_path = self.get_sql_path(file_name)
        return open(sql_path, 'r').read()

    def _extract_operation_from_sql(self, sql_str):
        """
        Return the operation (as a string) declared in the SQL string.
        """
        match = re.search(r'^([A-z]+)', sql_str, re.M)
        if match:
            past_tense = re.sub(r'E$', '', match.group())
            return '%sed' % past_tense.lower()
        else:
            return 'affected'

    def execute_custom_sql(self, file_name, params=None, composables=None):
        """
        Execute custom sql.

        Args:
            file_name (str): Name of .sql file.
            params (dict): Map of named placeholder in sql to parameter.
            composables (dict): Map of named placeholder in sql to psycopg2 sql Composable.

        Log the number of rows and operation performed.
        """
        # Get the path to the SQL file
        raw_sql = self.get_sql(file_name)

        # Compile with any composable variables
        if composables:
            composed_sql = sql.SQL(raw_sql).format(**composables)
        else:
            composed_sql = raw_sql

        # Open a database connection
        with connection.cursor() as cursor:
            # Run the SQL
            cursor.execute(composed_sql, params)
            # Get the row row_count
            row_count = cursor.rowcount

        # Log the result
        operation = self._extract_operation_from_sql(raw_sql)
        logger.debug('{} {} {}'.format(
            row_count,
            "row" + pluralize(row_count),
            operation
        ))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Abstract base models for campaign finance-related filings and transactions.
"""
from django.db import models
from calaccess_processed_filings.models.base import FilingBaseModel


class CampaignLoanItemBase(FilingBaseModel):
    """
    Abstract base model for loans received or made by campaign filers.

    These transactions are itemized on Schedules B (Parts 1 and 2) and H of
    Form 460 filings and stored in the LOAN_CD table.
    """
    line_item = models.IntegerField(
        verbose_name='line item',
        help_text="Line number of the filing form where the loan is "
                  "itemized (from LOAN_CD.LINE_ITEM)",
    )
    intermediary_title = models.CharField(
        verbose_name='intermediary title',
        max_length=10,
        blank=True,
        help_text='Name title of the intermediary (from LOAN_CD.INTR_NAMT)',
    )
    intermediary_lastname = models.CharField(
        verbose_name='intermediary lastname',
        max_length=200,
        blank=True,
        help_text='Last name of the intermediary or business name (from '
                  'LOAN_CD.INTR_NAML)',
    )
    intermediary_firstname = models.CharField(
        verbose_name='intermediary firstname',
        max_length=45,
        help_text='First name of the intermediary (from LOAN_CD.INTR_NAMF)',
    )
    intermediary_name_suffix = models.CharField(
        verbose_name='intermediary name suffix',
        max_length=10,
        blank=True,
        help_text='Name suffix of the intermediary (from LOAN_CD.INTR_NAMS)',
    )
    intermediary_city = models.CharField(
        verbose_name='intermediary city',
        max_length=30,
        blank=True,
        help_text='City of the intermediary (from LOAN_CD.INTR_CITY)',
    )
    intermediary_state = models.CharField(
        verbose_name='intermediary state',
        max_length=2,
        blank=True,
        help_text='State of the intermediary (from LOAN_CD.INTR_ST)',
    )
    intermediary_zip = models.CharField(
        verbose_name='intermediary zip',
        max_length=10,
        blank=True,
        help_text='Zip code (usually zip5, sometimes zip9) of the '
                  'intermediary (from LOAN_CD.INTR_ZIP4)',
    )
    interest_rate = models.CharField(
        verbose_name='interest rate',
        max_length=30,
        blank=True,
        help_text='Interest rate of the loan. This is sometimes expressed as a '
                  'decimal (e.g., 0.10) and other times as a percent (e.g., '
                  '10.0% (from LOAN_CD.LOAN_RATE)'
    )
    transaction_id = models.CharField(
        verbose_name='transaction id',
        max_length=20,
        help_text='Identifies a unique transaction across versions of the a '
                  'given Form 460 filing (from LOAN_CD.TRAN_ID)',
    )
    memo_reference_number = models.CharField(
        verbose_name='memo reference number',
        max_length=20,
        blank=True,
        help_text="A value assigned by the filer which refers to the item's"
                  "footnote in the TEXT_MEMO_CD table (from LOAN_CD."
                  "MEMO_REFNO)",
    )

    class Meta:
        """
        Model options.
        """
        abstract = True


class CampaignLoanReceivedItemBase(CampaignLoanItemBase):
    """
    Abstract base model for loans received by campaign filers.

    These transactions are itemized on Schedule B (Parts 1 and 2) of Form 460
    filings and stored in the LOAN_CD table.
    """
    LENDER_CODE_CHOICES = (
        ('COM', "Committee"),
        ('IND', "Individual"),
        ('OTH', "Other"),
        ('PTY', "Political Party"),
        ('RCP', "Recipient committee"),
        ('SCC', "Small Contributor Committee"),
        ('???', "Unknown value"),
    )
    lender_code = models.CharField(
        verbose_name='lender code',
        max_length=3,
        blank=True,
        choices=LENDER_CODE_CHOICES,
        help_text='Code describing the lender (from LOAN_CD.ENTITY_CD)',
    )
    lender_committee_id = models.CharField(
        verbose_name='lender committee id',
        max_length=9,
        blank=True,
        help_text="lender's filer identification number, if it is a "
                  "committee (from LOAN_CD.CMTE_ID)",
    )
    lender_title = models.CharField(
        verbose_name='lender title',
        max_length=10,
        blank=True,
        help_text="Name title of the lender (from LOAN_CD.LNDR_NAMT)",
    )
    lender_lastname = models.CharField(
        verbose_name='lender lastname',
        max_length=200,
        blank=True,
        help_text="Last name of the lender or business name (from LOAN_CD."
                  "LNDR_NAML)",
    )
    lender_firstname = models.CharField(
        verbose_name='lender firstname',
        max_length=45,
        blank=True,
        help_text="First name of the lender (from LOAN_CD.LNDR_NAMF)",
    )
    lender_name_suffix = models.CharField(
        verbose_name='lender name suffix',
        max_length=10,
        blank=True,
        help_text="Name suffix of the lender (from LOAN_CD.LNDR_NAMS)",
    )
    lender_city = models.CharField(
        verbose_name='lender city',
        max_length=30,
        blank=True,
        help_text='City of the lender (from LOAN_CD.LOAN_CITY)',
    )
    lender_state = models.CharField(
        verbose_name='lender state',
        max_length=2,
        blank=True,
        help_text='State of the lender (from LOAN_CD.LOAN_ST)',
    )
    lender_zip = models.CharField(
        verbose_name='lender zip',
        max_length=10,
        blank=True,
        help_text='Zip code (usually zip5, sometimes zip9) of the '
                  'lender (from LOAN_CD.LOAN_ZIP4)',
    )
    lender_employer = models.CharField(
        verbose_name='lender employer',
        max_length=200,
        blank=True,
        help_text='Employer of the lender (from LOAN_CD.LOAN_EMP)',
    )
    lender_occupation = models.CharField(
        verbose_name='lender occupation',
        max_length=60,
        blank=True,
        help_text='Occupation of the lender (from LOAN_CD.LOAN_OCC)',
    )
    lender_is_self_employed = models.BooleanField(
        verbose_name='lender is self employed',
        default=False,
        help_text='Indicates whether or not the lender is self-employed'
                  '(from LOAN_CD.LOAN_SELF)',
    )
    treasurer_title = models.CharField(
        verbose_name='treasurer title',
        max_length=10,
        blank=True,
        help_text="Name title of the lender committee's treasurer (from "
                  "LOAN_CD.TRES_NAMT)",
    )
    treasurer_lastname = models.CharField(
        verbose_name='treasurer lastname',
        max_length=200,
        blank=True,
        help_text="Last name of the lender committee's treasurer (from "
                  "LOAN_CD.TRES_NAML)",
    )
    treasurer_firstname = models.CharField(
        verbose_name='treasurer firstname',
        max_length=45,
        help_text="First name of the lender committee's treasurer (from "
                  "LOAN_CD.TRES_NAMF)",
    )
    treasurer_name_suffix = models.CharField(
        verbose_name='treasurer name suffix',
        max_length=10,
        blank=True,
        help_text="Name suffix of the lender committee's treasurer (from "
                  "LOAN_CD.TRES_NAMS)",
    )
    treasurer_city = models.CharField(
        verbose_name='treasurer city',
        max_length=30,
        blank=True,
        help_text="City of the lender committee's treasurer (from LOAN_CD."
                  "TRES_CITY)",
    )
    treasurer_state = models.CharField(
        verbose_name='treasurer state',
        max_length=2,
        blank=True,
        help_text="State of the lender committee's treasurer (from LOAN_CD."
                  "TRES_ST)",
    )
    treasurer_zip = models.CharField(
        verbose_name='treasurer zip',
        max_length=10,
        blank=True,
        help_text="Zip code (usually zip5, sometimes zip9) of the lender "
                  "committee's treasurer (from LOAN_CD.TRES_ZIP4)",
    )

    class Meta:
        """
        Model options.
        """
        abstract = True


class CampaignLoanMadeItemBase(CampaignLoanItemBase):
    """
    Abstract base model for loans made by campaign filers.

    These transactions are itemized on Schedule H of Form 460 filings and
    stored in the LOAN_CD table.
    """
    RECIPIENT_CODE_CHOICES = (
        ('COM', "Committee"),
        ('IND', "Individual"),
        ('OTH', "Other"),
        ('PTY', "Political Party"),
        ('RCP', "Recipient committee"),
        ('SCC', "Small Contributor Committee"),
        ('???', "Unknown value"),
    )
    recipient_code = models.CharField(
        verbose_name='recipient code',
        max_length=3,
        blank=True,
        choices=RECIPIENT_CODE_CHOICES,
        help_text='Code describing the recipient (from LOAN_CD.ENTITY_CD)',
    )
    recipient_committee_id = models.CharField(
        verbose_name='recipient committee id',
        max_length=9,
        blank=True,
        help_text="recipient's filer identification number, if it is a "
                  "committee (from LOAN_CD.CMTE_ID)",
    )
    recipient_title = models.CharField(
        verbose_name='recipient title',
        max_length=10,
        blank=True,
        help_text="Name title of the recipient (from LOAN_CD.LNDR_NAMT)",
    )
    recipient_lastname = models.CharField(
        verbose_name='recipient lastname',
        max_length=200,
        blank=True,
        help_text="Last name of the recipient or business name (from LOAN_CD."
                  "LNDR_NAML)",
    )
    recipient_firstname = models.CharField(
        verbose_name='recipient firstname',
        max_length=45,
        blank=True,
        help_text="First name of the recipient (from LOAN_CD.LNDR_NAMF)",
    )
    recipient_name_suffix = models.CharField(
        verbose_name='recipient name suffix',
        max_length=10,
        blank=True,
        help_text="Name suffix of the recipient (from LOAN_CD.LNDR_NAMS)",
    )
    recipient_city = models.CharField(
        verbose_name='recipient city',
        max_length=30,
        blank=True,
        help_text='City of the recipient (from LOAN_CD.LOAN_CITY)',
    )
    recipient_state = models.CharField(
        verbose_name='recipient state',
        max_length=2,
        blank=True,
        help_text='State of the recipient (from LOAN_CD.LOAN_ST)',
    )
    recipient_zip = models.CharField(
        verbose_name='recipient zip',
        max_length=10,
        blank=True,
        help_text='Zip code (usually zip5, sometimes zip9) of the '
                  'recipient (from LOAN_CD.LOAN_ZIP4)',
    )
    recipient_employer = models.CharField(
        verbose_name='recipient employer',
        max_length=200,
        blank=True,
        help_text='Employer of the recipient (from LOAN_CD.LOAN_EMP)',
    )
    recipient_occupation = models.CharField(
        verbose_name='recipient occupation',
        max_length=60,
        blank=True,
        help_text="Occupation of the recipient (from LOAN_CD.LOAN_OCC). Note "
                  "that, in some cases, the value seems to actually be the "
                  "recipient's committee ID.",
    )
    recipient_is_self_employed = models.BooleanField(
        verbose_name='recipient is self employed',
        default=False,
        help_text='Indicates whether or not the recipient is self-employed'
                  '(from LOAN_CD.LOAN_SELF)',
    )
    treasurer_title = models.CharField(
        verbose_name='treasurer title',
        max_length=10,
        blank=True,
        help_text="Name title of the recipient committee's treasurer (from "
                  "LOAN_CD.TRES_NAMT)",
    )
    treasurer_lastname = models.CharField(
        verbose_name='treasurer lastname',
        max_length=200,
        blank=True,
        help_text="Last name of the recipient committee's treasurer (from "
                  "LOAN_CD.TRES_NAML)",
    )
    treasurer_firstname = models.CharField(
        verbose_name='treasurer firstname',
        max_length=45,
        help_text="First name of the recipient committee's treasurer (from "
                  "LOAN_CD.TRES_NAMF)",
    )
    treasurer_name_suffix = models.CharField(
        verbose_name='treasurer name suffix',
        max_length=10,
        blank=True,
        help_text="Name suffix of the recipient committee's treasurer (from "
                  "LOAN_CD.TRES_NAMS)",
    )
    treasurer_city = models.CharField(
        verbose_name='treasurer city',
        max_length=30,
        blank=True,
        help_text="City of the recipient committee's treasurer (from LOAN_CD."
                  "TRES_CITY)",
    )
    treasurer_state = models.CharField(
        verbose_name='treasurer state',
        max_length=2,
        blank=True,
        help_text="State of the recipient committee's treasurer (from LOAN_CD."
                  "TRES_ST)",
    )
    treasurer_zip = models.CharField(
        verbose_name='treasurer zip',
        max_length=10,
        blank=True,
        help_text="Zip code (usually zip5, sometimes zip9) of the recipient "
                  "committee's treasurer (from LOAN_CD.TRES_ZIP4)",
    )

    class Meta:
        """
        Model options.
        """
        abstract = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing data from Campaign Disclosure Statements (Form 460).
"""
from django.db import models
from calaccess_processed_filings.models.campaign import CampaignFinanceFilingBase


class Form460FilingBase(CampaignFinanceFilingBase):
    """
    Base and abstract model for Form 460 filings.
    """
    statement_type = models.CharField(
        max_length=50,
        verbose_name='statement type',
        help_text='Type of statement, e.g., "Quarterly", "Semi-Annual", Pre-'
                  'Election (from CVR_CAMPAIGN_DISCLOSURE.STMT_TYPE)',
    )
    from_date = models.DateField(
        verbose_name='from date',
        db_index=True,
        null=False,
        help_text="The first date of the filing period covered by the statement "
                  "(from CVR_CAMPAIGN_DISCLOSURE.FROM_DATE)",
    )
    thru_date = models.DateField(
        verbose_name='thru date',
        db_index=True,
        null=False,
        help_text="The last date of the filing period covered by the statement "
                  "(from CVR_CAMPAIGN_DISCLOSURE.THRU_DATE)",
    )
    monetary_contributions = models.IntegerField(
        verbose_name='monetary contributions',
        null=True,
        help_text="Total monetary contributions (from line 1, column A)",
    )
    loans_received = models.IntegerField(
        verbose_name='loans received',
        null=True,
        help_text="Total loans received (from line 2, column A)",
    )
    subtotal_cash_contributions = models.IntegerField(
        verbose_name='subtotal cash contributions',
        null=True,
        help_text="Monetary contributions and loans received combined (from "
                  "line 3, column A)",
    )
    nonmonetary_contributions = models.IntegerField(
        verbose_name='nonmonetary contributions',
        null=True,
        help_text="Non-monetary contributions (from line 4, column A)",
    )
    total_contributions = models.IntegerField(
        verbose_name='total contributions',
        null=True,
        help_text="Total contributions (from line 5, column A)",
    )
    payments_made = models.IntegerField(
        verbose_name='payments made',
        null=True,
        help_text="Payments made (from line 6, column A)",
    )
    loans_made = models.IntegerField(
        verbose_name='loans made',
        null=True,
        help_text="Loans made (from line 7, column A)",
    )
    subtotal_cash_payments = models.IntegerField(
        verbose_name='subtotal cash payments',
        null=True,
        help_text="Sub-total of cash payments (from line 8, column A)",
    )
    unpaid_bills = models.IntegerField(
        verbose_name='unpaid bills',
        null=True,
        help_text="Unpaid bills / accrued expenses (from line 9, column A)",
    )
    nonmonetary_adjustment = models.IntegerField(
        verbose_name='nonmonetary adjustment',
        null=True,
        help_text="Non-monetary adjustment (from line 10, column A), which is "
                  "equal to the total of non-monetary contributions",
    )
    total_expenditures_made = models.IntegerField(
        verbose_name='total expenditures made',
        null=True,
        help_text="Total expenditures made (from line 11, column A)",
    )
    beginning_cash_balance = models.IntegerField(
        verbose_name='begin cash balance',
        null=True,
        help_text="Beginning cash balance (from line 12), which is equal to "
                  "the Ending Cash Balance (line 16) reported on the summary "
                  "page of the previous Form 460 filing"
    )
    cash_receipts = models.IntegerField(
        verbose_name='cash receipts',
        null=True,
        help_text="Cash receipts (from line 13)",
    )
    miscellaneous_cash_increases = models.IntegerField(
        verbose_name='miscellaneous cash increases',
        null=True,
        help_text="Miscellaneous cash increases (from line 14)",
    )
    cash_payments = models.IntegerField(
        verbose_name='cash payments',
        null=True,
        help_text="Cash payments (from line 15)",
    )
    ending_cash_balance = models.IntegerField(
        verbose_name='ending cash balance',
        null=True,
        help_text="Ending cash balance (from line 16)",
    )
    loan_guarantees_received = models.IntegerField(
        verbose_name='loan guarantees received',
        null=True,
        help_text="Loan guarantees received (from line 17)",
    )
    cash_equivalents = models.IntegerField(
        verbose_name='cash equivalents',
        null=True,
        help_text="Cash equivalents (from line 18), which includes investments "
                  "that can't be readily converted to cash, such as outstanding "
                  "loans the committee has made to others"
    )
    outstanding_debts = models.IntegerField(
        verbose_name='outstanding debts',
        null=True,
        help_text="Outstanding debts on loans owed by the committee (from line "
                  "19)",
    )

    class Meta:
        """
        Model options.
        """
        app_label = 'calaccess_processed_filings'
        abstract = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Proxy models for augmenting our source data tables with methods useful for processing.
"""
from __future__ import unicode_literals

# Models
from opencivicdata.core.models import (
    Membership,
    Organization,
    OrganizationIdentifier,
    OrganizationName,
)
from .people import OCDPersonProxy
from calaccess_processed.proxies import OCDProxyModelMixin

# Managers
from calaccess_processed.managers import BulkLoadSQLManager
from calaccess_processed_elections.managers import (
    OCDOrganizationManager,
    OCDMembershipManager
)

# Logging
import logging
logger = logging.getLogger(__name__)


class OCDOrganizationProxy(Organization, OCDProxyModelMixin):
    """
    A proxy on the OCD Organization model with helper methods.
    """
    objects = OCDOrganizationManager()

    copy_to_fields = (
        ('id',),
        ('name',),
        ('image',),
        ('parent_id',),
        ('jurisdiction_id',),
        ('classification',),
        ('founding_date',),
        ('dissolution_date',),
        ('created_at',),
        ('updated_at',),
        ('extras',),
        ('locked_fields',),
    )

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_elections"
        proxy = True


class OCDOrganizationIdentifierProxy(OrganizationIdentifier, OCDProxyModelMixin):
    """
    A proxy on the OCD OrganizationIdentifier model with helper methods.
    """
    objects = BulkLoadSQLManager()

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_elections"
        proxy = True


class OCDOrganizationNameProxy(OrganizationName, OCDProxyModelMixin):
    """
    A proxy on the OCD OrganizationName model with helper methods.
    """
    objects = BulkLoadSQLManager()

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_elections"
        proxy = True


class OCDMembershipProxy(Membership, OCDProxyModelMixin):
    """
    A proxy on the OCD Membership model with helper methods.
    """
    objects = OCDMembershipManager()

    copy_to_fields = (
        ('id',),
        ('organization_id',),
        ('person_id',),
        ('person_name',),
        ('post_id',),
        ('on_behalf_of_id',),
        ('label',),
        ('role',),
        ('start_date',),
        ('end_date',),
        ('created_at',),
        ('updated_at',),
        ('extras',),
        ('locked_fields',),
    )

    @property
    def person_proxy(self):
        """
        Returns an OCDPersonProxy instance linked to the Membership.
        """
        person = self.person
        person.__class__ = OCDPersonProxy
        return person

    class Meta:
        """
        Make this a proxy model.
        """
        app_label = "calaccess_processed_elections"
        proxy = True
